﻿using Microsoft.EntityFrameworkCore;
using MotorPlantContracts.BindingModels;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using MotorPlantDatabaseImplement.Models;


namespace MotorPlantDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new MotorPlantDatabase();
            return context.Orders.Include(x => x.Engine).Select(x => x.GetViewModel).ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new MotorPlantDatabase();
            return context.Orders
                    .Include(x => x.Engine)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new MotorPlantDatabase();
            return context.Orders.Include(x => x.Engine)
                .FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new MotorPlantDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                      .Include(x => x.Engine)
                      .FirstOrDefault(x => x.Id == newOrder.Id)
                      ?.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new MotorPlantDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders
                         .Include(x => x.Engine)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new MotorPlantDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Orders
                                           .Include(x => x.Engine)
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
    }
}
