﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.ViewModels;
using MotorPlantDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace MotorPlantDatabaseImplement.Models
{
    public class Engine : IEngineModel
    {
        public int Id { get; set; }
        [Required]
        public string EngineName { get; set; } = string.Empty;
        [Required]
        public double Price { get; set; }
        private Dictionary<int, (IComponentModel, int)>? _engineComponents = null;
        [NotMapped]
        public Dictionary<int, (IComponentModel, int)> EngineComponents
        {
            get
            {
                if (_engineComponents == null)
                {
                    _engineComponents = Components.ToDictionary(recDC => recDC.ComponentId, recDC => (recDC.Component as IComponentModel, recDC.Count));
                }
                return _engineComponents;
            }
        }
        [ForeignKey("EngineId")]
        public virtual List<EngineComponent> Components { get; set; } = new();
        [ForeignKey("EngineId")]
        public virtual List<Order> Orders { get; set; } = new();
        public static Engine Create(MotorPlantDatabase context, EngineBindingModel model)
        {
            return new Engine()
            {
                Id = model.Id,
                EngineName = model.EngineName,
                Price = model.Price,
                Components = model.EngineComponents.Select(x => new EngineComponent
                {
                    Component = context.Components.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }
        public void Update(EngineBindingModel model)
        {
            EngineName = model.EngineName;
            Price = model.Price;
        }
        public EngineViewModel GetViewModel => new()
        {
            Id = Id,
            EngineName = EngineName,
            Price = Price,
            EngineComponents = EngineComponents
        };
        public void UpdateComponents(MotorPlantDatabase context, EngineBindingModel model)
        {
            var engineComponents = context.EngineComponents.Where(rec => rec.EngineId == model.Id).ToList();
            if (engineComponents != null && engineComponents.Count > 0)
            {
                context.EngineComponents.RemoveRange(engineComponents.Where(rec => !model.EngineComponents.ContainsKey(rec.ComponentId)));
                context.SaveChanges();
                foreach (var updateComponent in engineComponents)
                {
                    updateComponent.Count = model.EngineComponents[updateComponent.ComponentId].Item2;
                    model.EngineComponents.Remove(updateComponent.ComponentId);
                }
                context.SaveChanges();
            }
            var engine = context.Engines.First(x => x.Id == Id);
            foreach (var pc in model.EngineComponents)
            {
                context.EngineComponents.Add(new EngineComponent
                {
                    Engine = engine,
                    Component = context.Components.First(x => x.Id == pc.Key),
                    Count = pc.Value.Item2
                });
                context.SaveChanges();
            }
            _engineComponents = null;
        }
    }
}
